#!/usr/bin/env python
#-*- coding: UTF-8 -*-

# notify-send 'Hello world!' 'This is an example notification.' --icon=dialog-information

import sys
import os
import os.path
import threading
import dbus
import dbus.glib
import gobject
import wx
import wx.html
import time
import __builtin__
__builtin__.__dict__['_'] = wx.GetTranslation
import logging
import logging.handlers
import webbrowser
from dbus.mainloop.glib import DBusGMainLoop
from   .nofyhistmisc import *

try:
    import wx.lib.evtmgr as evtmgr
    import wx.lib.intctrl
except:    
    class PanicApp(wx.App):
        def OnInit(self):
            print('\n< < <  P A N I C  > > >\n')
            print("Can not start NofyHist because the lib 'intctrl' is missing!")
            print('Check your installation. The system reports, that you are')
            print(WXVERSION, '- but it has to be 2.8 or better!')
            print()
            print('Please install wxPython 2.8 (package python-wxgtk2.8 on')
            print('Ubuntu / Kubuntu).')
            return True
            
    app = PanicApp(redirect = True)
    app.MainLoop()
   
from . import nofyhistabout
from . import nofyhistsystray
from . import nofyhistcustbar
from . import nofyhistupdate
from . import nofyhistprefs
from . import nofyhistinstall
from . import nofyhistfind
from . import nofyhistexclude
from . import nofyhistpopup

from   .configobj import ConfigObj
from   .singleinstance import SingleInstance


class MessagesHistory(object):
    def __init__(self, config):
        self.messages = []
        self.config = config
    #-------------------------------------------------------------------------------------------
            
    def newMessage(self, msgargs):
        curtime = time.strftime('%Y-%m-%d %H:%M:%S')
        self.messages.insert(0, (msgargs[0], msgargs[3], msgargs[4], curtime))
        self.checkLimit()
    #-------------------------------------------------------------------------------------------
    
    def checkLimit(self):
        limit = int(self.config['Misc']['limit'])
        while len(self.messages) > limit:
            del self.messages[len(self.messages) - 1]
    #-------------------------------------------------------------------------------------------
        
    def getMessage(self, index):
        return self.messages[index]
    #-------------------------------------------------------------------------------------------
    
    def getMessages(self):
        return self.messages
# - class MessagesHistory(object):--------------------------------------------------------------
    
           
class NofyHistFrame(wx.Frame):   
    ID_LOGWIN = wx.NewId()
    ID_SYSTRAY = wx.NewId()
    ID_EXIT = wx.NewId()
    ID_WEBSITE = wx.NewId()
    ID_UPDATE = wx.NewId()
    ID_INSTALL = wx.NewId()
    ID_ABOUT = wx.NewId()
    ID_HELP = wx.NewId()
    ID_PREFS = wx.NewId()
    ID_FIND = wx.NewId()
    ID_EXCLUDE = wx.NewId()
    ID_CLEAR_ALL = wx.NewId()
    ID_LISTCTRL = wx.NewId()
    

    def __init__(self, parent, id, title, instance):
        self.instance = instance        # this object should live until termination
        try:
            self.handleConfigData('r')
            self.initconfig = False
        except:
            self.handleConfigData('i')
            self.initconfig = True
        
        # init the DBUS
        DBusGMainLoop(set_as_default=True)
        bus = dbus.SessionBus()
        matchstr = "interface='org.freedesktop.Notifications',member='Notify',eavesdrop='true'"
        bus.add_match_string(matchstr)
        bus.add_message_filter(self.msg_filter)        
        gobject.threads_init() 
        dbus.glib.threads_init()
        
        self.messageshistory = MessagesHistory(self.config)
                
        langindex = self.config['Gui']['language'].index(self.config['Gui']['curlang'])
        wxlanguages = (wx.LANGUAGE_DEFAULT, wx.LANGUAGE_ENGLISH, wx.LANGUAGE_GERMAN)
        self.mylocale = wx.Locale(wxlanguages[langindex])
        self.mylocale.AddCatalogLookupPathPrefix(LOCALEDIR)
        self.mylocale.AddCatalog(TITLE)        
        
        windowpos = (wx.Point(int(self.config['Window']['posx']),
                               int(self.config['Window']['posy'])))
        windowsize = (wx.Size(int(self.config['Window']['sizex']),
                              int(self.config['Window']['sizey'])))
        wx.Frame.__init__(self, parent, id, title, pos = windowpos, size = windowsize)  

        if self.config['Misc']['systray'] == 'True':
            self.Bind(wx.EVT_CLOSE, self.onSysTray)
        else:
            self.Bind(wx.EVT_CLOSE, self.onCloseWindow)

        iconfile = ICONPATH + os.sep + APPICON
        if os.path.exists(iconfile):
            image = wx.Image(iconfile, wx.BITMAP_TYPE_PNG)
            image.Rescale(SCALE, SCALE, wx.IMAGE_QUALITY_NORMAL)
            self.icon = wx.IconFromBitmap(image.ConvertToBitmap())
            self.SetIcon(self.icon)
        else:
            self.icon = None

        logfilename = self.inifilename.replace('.ini', '-Action.log')
        logpath = os.path.join(HOMEDIR, logfilename)
        self.logger = logging.getLogger('ActionLog')
        self.logger.setLevel(logging.INFO)
        handler = logging.handlers.RotatingFileHandler(logpath, 'a', ACTION_LOG_SIZE, 4)
        formatter = logging.Formatter("%(asctime)s -- %(message)s", "%Y-%m-%d %H:%M:%S")
        handler.setFormatter(formatter)        
        self.logger.addHandler(handler)
        
        self.initMenu()
        self.sb = nofyhistcustbar.CustomStatusBar(self)
        self.SetStatusBar(self.sb)        

        collist = self.config['Misc']['hlcolor']
        self.hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        
        self.excludelist = self.config['Misc']['exclude']
        if self.config['Gui']['grid'] == 'True':      
            style = wx.LC_REPORT | wx.BORDER_SUNKEN | wx.LC_HRULES | wx.LC_VRULES
        else:
            style = wx.LC_REPORT | wx.BORDER_SUNKEN    
        self.listwin = wx.ListCtrl(self, self.ID_LISTCTRL, style = style)
        self.listwin.InsertColumn(0, _('Sender'))
        self.listwin.InsertColumn(1, _('Header'))
        self.listwin.InsertColumn(2, _('Message body'))
        self.listwin.SetColumnWidth(0, int(self.config['Window']['lwcol0']))
        self.listwin.SetColumnWidth(1, int(self.config['Window']['lwcol1']))
        self.listwin.SetColumnWidth(2, int(self.config['Window']['lwcol2']))
        self.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.onShowItemInfo, self.listwin)

        self.myfont = wx.Font(int(self.config['Gui']['fsize']), wx.DEFAULT, wx.NORMAL, wx.NORMAL, False, self.config['Gui']['font'])
        self.listwin.SetFont(self.myfont)
        self.Bind(wx.EVT_LIST_COL_END_DRAG, self.onColSize, self.listwin)        

        if PIL_INSTALLED:
            try:
                self.icon = Image.open(ICONPATH + os.sep + 'count.png')
            except:
                pass
        self.tbicon = nofyhistsystray.SysTrayIcon(self, self.icon, FONT, TITLE)          
        self.initContextMenu()

        self.mainmenu.Enable(self.ID_INSTALL, False)

        self.updateListwin()
             
        if self.config['Misc']['update'] == 'True':
            self.updatecheck = True
            self.updatetimer = wx.Timer(self, -1) 
            self.Bind(wx.EVT_TIMER, self.onUpdate, self.updatetimer) 
            self.updatetimer.Start(8000, oneShot = True)
        else:
            self.updatecheck = False            
        self.logger.info(_('%s started ...') % (VERSION))
        
        self.minimizetimer = wx.Timer(self, -1)
        self.Bind(wx.EVT_TIMER, self.onSysTray, self.minimizetimer)
        if self.config['Misc']['startmin'] == 'True':
            self.minimizetimer.Start(250, oneShot = True)
            
        if self.initconfig:
            self.initimer = wx.Timer(self, -1) 
            self.Bind(wx.EVT_TIMER, self.onInitIni, self.initimer) 
            self.initimer.Start(1000, oneShot = True)
            
        self.delaftertimer = wx.Timer(self, -1) 
        self.Bind(wx.EVT_TIMER, self.onDeleteAfter, self.delaftertimer) 
        self.delaftertimer.Start(1000 * 600, oneShot = False)  # every 10 minutes
    #-------------------------------------------------------------------------------------------

    def initMenu(self):
        self.mainmenu = wx.MenuBar()         
        # File menu
        menu = wx.Menu()                     
        menu.AppendSeparator() 
        item = wx.MenuItem(menu, self.ID_EXIT, _('E&xit NofyHist\tCTRL-X'), _('exit the application'))
        item.SetBitmap(loadIcon('exit.png'))                 
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onCloseWindow, id = self.ID_EXIT)
        self.mainmenu.Append(menu, _('&File'))

        # Options menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_CLEAR_ALL, _('Clear &all\tCTRL-A'), _('clear all items'))
        item.SetBitmap(loadIcon('clearall.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onClearAll, id = self.ID_CLEAR_ALL)
        menu.AppendSeparator()
        
        item = wx.MenuItem(menu, self.ID_FIND, _('Find\tF3'), _('find item'))
        item.SetBitmap(loadIcon('search.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onFindItem, id = self.ID_FIND)
        
        item = wx.MenuItem(menu, self.ID_EXCLUDE, _('&Exclude filter\tCTRL-E'), _('edit exclude filter'))
        item.SetBitmap(loadIcon('exclude.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onExcludeFilter, id = self.ID_EXCLUDE)   
        
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_PREFS, _('&Preferences\tCTRL-P'), _('set preferences'))
        item.SetBitmap(loadIcon('prefs.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onPreferences, id = self.ID_PREFS)        
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_SYSTRAY, _('S&ystray icon\tCTRL-Y'), _('minimize app to systray'))
        item.SetBitmap(loadIcon('systray.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onSysTray, id = self.ID_SYSTRAY)
        self.mainmenu.Append(menu, _('&Options'))
        
        # Help menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_HELP, _('&Help (online)\tF1'), _('online html help docs'))
        item.SetBitmap(loadIcon('help.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onHandbook, id = self.ID_HELP)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_WEBSITE, _('NofyHist &Website\tCTRL-W'), _('NofyHist Website'))
        item.SetBitmap(loadIcon('website.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onWebsite, id = self.ID_WEBSITE)
        menu.AppendSeparator()     
        item = wx.MenuItem(menu, self.ID_UPDATE, _('NofyHist U&pdates\tCTRL-P'), _('download newest version'))
        item.SetBitmap(loadIcon('update.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onUpdate, id = self.ID_UPDATE)                
        item = wx.MenuItem(menu, self.ID_INSTALL, _('install NofyHist Updates'), _('install newest version'))
        item.SetBitmap(loadIcon('update.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onInstallUpdate, id = self.ID_INSTALL)                
        menu.AppendSeparator()        
        item = wx.MenuItem(menu, self.ID_ABOUT, _('About NofyHist\tSHIFT-CTRL-A'), _('about the application'))
        item.SetBitmap(loadIcon('about.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onAbout, id = self.ID_ABOUT)
        self.mainmenu.Append(menu, _('&Help'))

        self.SetMenuBar(self.mainmenu)
    #-------------------------------------------------------------------------------------------
        
    def initContextMenu(self):
        self.popupID0 = wx.NewId()  
        self.popupID1 = wx.NewId()          
        self.Bind(wx.EVT_MENU, self.onPreferences, id = self.popupID0)
        self.Bind(wx.EVT_MENU, self.onSysTray, id = self.popupID1)
    #-------------------------------------------------------------------------------------------
    
    def onContext(self, event):
        """ context menu handler """
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.popupID5, _('Pre&ferences\tCTRL-F'))
        item.SetBitmap(loadIcon('prefs.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.popupID6, _('S&ystray icon\tCTRL-Y'))
        item.SetBitmap(loadIcon('systray.png'))
        menu.AppendItem(item)        
        self.PopupMenu(menu)
        menu.Destroy()
    #-------------------------------------------------------------------------------------------      

    def onColSize(self, event):
        for i in range(self.listwin.GetColumnCount()):
            self.config['Window']['lwcol%i' % i] = self.listwin.GetColumnWidth(i)
    #-------------------------------------------------------------------------------------------
        
    def onColSize(self, event):
        for i in range(self.listwin.GetColumnCount()):
            self.config['Window']['lwcol%i' % i] = self.listwin.GetColumnWidth(i)
    #-------------------------------------------------------------------------------------------

    def msg_filter(self, bus, msg):
        args = msg.get_args_list()
        if len(args) < 5: return
        if not args[0] in self.excludelist:
            self.messageshistory.newMessage(args)
            lastm = self.messageshistory.getMessage(0)
            s = 'Sender: %s, Header: %s, Body: %s' % (lastm[0], lastm[1], lastm[2])
            s = s.replace('\n', ' ')
            self.logger.info(s)
            self.updateListwin(reuse = True)            
    #-------------------------------------------------------------------------------------------

    def updateListwin(self, reuse = False, hlindexes = []):
        if reuse:
            hlindexes = self.hlindexes
        else:
            self.hlindexes = hlindexes
        self.listwin.DeleteAllItems()
        
        messages = self.messageshistory.getMessages()
        for index, message in enumerate(messages):            
            item = self.listwin.InsertImageStringItem(index, message[0], 0)
            self.listwin.SetStringItem(index, 1, message[1])
            self.listwin.SetStringItem(index, 2, message[2])
            if index in hlindexes:
                self.listwin.SetItemBackgroundColour(index, self.hlcolor)
        
        infotext = VERSION
        count = len(messages)
        self.tbicon.setTitle('%s\n%d Messages' % (infotext, count), count)
        excludelist = (', ').join(self.excludelist)
        self.sb.SetStatusText('M: %d | F: %d' %(count, len(hlindexes)), 1)
        self.sb.SetStatusText(_('Exclude filter: %s') % (excludelist), 2)
        self.sb.rePosition()
        self.listwin.Update()
    #-------------------------------------------------------------------------------------------

    def onDeleteAfter(self, event):
        delta = float(self.config['Misc']['delafter']) * 3600.0
        curtime = time.strftime('%Y-%m-%d %H:%M:%S')
        curtimesec = time.mktime(time.strptime(curtime,"%Y-%m-%d %H:%M:%S"))
        messages = self.messageshistory.getMessages()
        for index in range (len(messages)-1, 0, -1):
            mtimesec = time.mktime(time.strptime(messages[index][3],"%Y-%m-%d %H:%M:%S"))
            if curtimesec - mtimesec >= delta:
                del messages[index]
        self.updateListwin(reuse = True)
    #-------------------------------------------------------------------------------------------

    def onShowItemInfo(self, event):
        message = self.messageshistory.getMessage(event.m_itemIndex)
        if message:
            s1 = 'Timestamp:    %s' % message[3]
            s2 = 'Sender:       %s' % message[0]
            s3 = 'Header:       %s' % message[1]
            
            splitline = message[2].split('\n')
            s4 = 'Message body: '
            for linepart in splitline:
                s4 = s4 + linepart + '\n              '
            s4 = s4[:-15]
            
            label = '%s\n%s\n%s\n%s' % (s1, s2, s3, s4)
            win = nofyhistpopup.PopupWin(self.listwin, label)
            
            pos = self.listwin.ClientToScreen((0,0))  # pos is the left top edge of the Backup task win
            sz = self.listwin.GetSize()               # sz is the size of the Backup task win
            win.Position((pos[0] + sz[0] / 2, pos[1] - sz[1] / 2), (0, sz[1]))       
            win.Popup()
    #-------------------------------------------------------------------------------------------

    def onClearAll(self, event):
        dlg = wx.MessageDialog(self, _('Clear all notification history entries?'), TITLE + _(' - Clear notification history'), wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)
        if dlg.ShowModal() == wx.ID_YES:
            self.messageshistory = MessagesHistory(self.config)
            self.updateListwin()
            self.logger.info(_('Clear - all messages deleted'))
    #-------------------------------------------------------------------------------------------

    def onFindItem(self, event):
        dlg = nofyhistfind.FindDialog(self, self.config, self.messageshistory.getMessages(), self.updateListwin)
        if dlg.ShowModal() == wx.ID_OK:
            self.config['Misc']['paramhist'] = dlg.getValue()
        else:
            self.updateListwin()
    #-------------------------------------------------------------------------------------------

    def onExcludeFilter(self, event):
        dlg = nofyhistexclude.ExcludeDialog(self, self.config, self.messageshistory.getMessages())
        dlg.ShowModal()
        self.excludelist = self.config['Misc']['exclude']
        messages = self.messageshistory.getMessages()
        for index in range (len(messages)-1, -1, -1):
            if messages[index][0] in self.excludelist:
                del messages[index]
        self.updateListwin()
    #-------------------------------------------------------------------------------------------

    def onPreferences(self, event):
        self.handleConfigData('s')
        delafter = self.config['Misc']['delafter']
        prefdlg = nofyhistprefs.PrefsDialog(self, self.hlcolor, self.inifilename)
        if prefdlg.ShowModal() == wx.ID_OK:
            self.config = prefdlg.getNewConfig()
            self.handleConfigData('s')
            self.logger.info(_('Preferences - new values set'))
        else:
            self.handleConfigData('r')
        prefdlg.Destroy()
        self.myfont = wx.Font(int(self.config['Gui']['fsize']),
                              wx.DEFAULT, wx.NORMAL, wx.NORMAL, False,
                              self.config['Gui']['font'])
        self.listwin.SetFont(self.myfont)
        collist = self.config['Gui']['hlcolor']
        self.hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        self.messageshistory.checkLimit()
                
        self.updateListwin(reuse = True)
        self.Unbind(wx.EVT_CLOSE)
        if self.config['Misc']['systray'] == 'True':
            self.Bind(wx.EVT_CLOSE, self.onSysTray)
        else:
            self.Bind(wx.EVT_CLOSE, self.onCloseWindow)            
    #-------------------------------------------------------------------------------------------

    def onAbout(self, event):
        dlg = nofyhistabout.AboutDialog(self)
        dlg.ShowModal()
        self.updateListwin()
    #-------------------------------------------------------------------------------------------

    def setWindowDefaults(self):
        self.config['Window']['posx'] = 0
        self.config['Window']['posy'] = 0
        self.config['Window']['sizex'] = 500
        self.config['Window']['sizey'] = 270
        self.config['Window']['lwcol0'] = 95
        self.config['Window']['lwcol1'] = 135
        self.config['Window']['lwcol2'] = 305
        self.config['Window']['findposx'] = 100
        self.config['Window']['findposy'] = 100
        self.config['Window']['findsizex'] = 200
        self.config['Window']['findsizex'] = 200
    #-------------------------------------------------------------------------------------------
        
    def handleConfigData(self, rw = 'r'):
        if rw == 'r' or rw == 'i':               # read  / init ini file
            inipath = HOMEDIR
            if len(sys.argv) > 1:
                argument = sys.argv[1].lower()
                if argument.startswith('-i='):
                    inipath = sys.argv[1][3:]
            self.inifilename = os.path.join(inipath, TITLE + '.ini')
            self.config = ConfigObj(self.inifilename, encoding = 'UTF8')
            if rw == 'i' or 'Version' not in self.config:       # new, empty ini file (Python 3)
                # initialize all values
                self.config.initial_comment = ['NofyHist preferences file']
                self.config['Version'] = {}
                self.config['Version']['vernum'] = VERNUM
                self.config['Version']['startup'] = ''
                self.config['Version']['shutdown'] = ''
                self.config['Window'] = {}
                self.setWindowDefaults()
                self.config['Gui'] = {}
                self.config['Gui']['grid'] = 'True'
                self.config['Gui']['font'] = wx.SystemSettings_GetFont(wx.SYS_ANSI_VAR_FONT).GetFaceName()
                self.config['Gui']['fsize'] = '9'                
                self.config['Gui']['norm'] = 'True'               
                self.config['Gui']['language'] = ('Default', 'English', 'German')
                self.config['Gui']['curlang'] = 'Default'
                self.config['Misc'] = {}
                self.config['Misc']['paramhist'] = ['']
                self.config['Misc']['exclude'] = ['Example']
                self.config['Misc']['actionlog'] = 'True'
                self.config['Misc']['update'] = 'False'
                self.config['Misc']['startmin'] = 'False'
                self.config['Misc']['systray'] = 'True'
                self.config['Misc']['hlcolor'] = (194, 226, 194)
                self.config['Misc']['hiddendirs'] = 'True'
                self.config['Misc']['limit'] = '25'
                self.config['Misc']['delafter'] = '24'
                self.config.write()
                        
            self.config['Version']['startup'] = time.strftime('%Y-%m-%d %H:%M:%S') 
        else:          # write ini file rw = w or rw = s
            self.config['Version']['vernum'] = VERNUM
            windowrect = self.GetRect()
            self.config['Window']['posx'] = windowrect[0]
            self.config['Window']['posy'] = windowrect[1]
            self.config['Window']['sizex'] = windowrect[2]
            self.config['Window']['sizey'] = windowrect[3]
            if rw == 'w':
                self.config['Version']['shutdown'] = time.strftime('%Y-%m-%d %H:%M:%S')                        
            self.config.write()
    #-------------------------------------------------------------------------------------------

    def readUrl(self):    
        if sys.version < '3':
            import urllib2
        else:
            import urllib.request, urllib.error, urllib.parse

        try:
            if sys.version < '3':
                f = urllib2.urlopen(URL_DOWNLOADWEBPAGE, timeout = 5)
            else:
                f = urllib.request.urlopen(URL_DOWNLOADWEBPAGE, timeout = 5)
            html = f.read()
            f.close()
        except:
            return            
        
        versions = []
        while len(html):
            fpos = html.find('NofyHist-')
            html = html[fpos:]
            version = html[9:13]
            if html[13:17] == '.zip':
                versions.append(version)
            html = html[24:]
            
        versions.sort(reverse = True)     
        self.versions = versions
    #-------------------------------------------------------------------------------------------

    def onUpdate(self, event):
        # check for new program release
        self.versions = -1
        self.logger.info(_('Update - try to connect to NofyHist website'))

        updatethread = threading.Thread(target = self.readUrl, args = ())
        updatethread.start()
        while updatethread.isAlive():
            try:
                wx.SafeYield(self)
            except:
                pass
        
        if type(self.versions) == int:
            dlg = wx.MessageDialog(self, _('unable to connect to the NofyHist website!'), TITLE + _(' - find NofyHist update'), wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            return            
        
        self.logger.info(_('Update - successfully connected to NofyHist website'))  
        if len(self.versions):
            version = self.versions[0]
            if version > VERNUM[:4]:
                dlg = nofyhistupdate.UpdateDialog(self, _("a newer version (%s) of NofyHist is available!") % (version))
                doit = dlg.ShowModal()
                self.mainmenu.Enable(self.ID_INSTALL, True)
                self.newrelease = '%s/%s-%s.zip' % (URL_DOWNLOADFOLDER, TITLE, version)
                if doit == wx.ID_OK:
                    self.onInstallUpdate()
                return
        else:
            self.logger.info(_("Update check - cannot read information from NofyHist website"))
            return
            
        if self.updatecheck:
            self.updatecheck = False
        else:    
            dlg = wx.MessageDialog(self, _('the installed version of NofyHist is up to date!'), TITLE + _(' - find update'), wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
        self.logger.info(_("Update check - the installed version of NofyHist is up to date"))
    #-------------------------------------------------------------------------------------------

    def onInstallUpdate(self, event = None):
        # install new program release
        dlg = nofyhistinstall.InstallDialog(self, self.config, self.logger.info, self.newrelease)
        if dlg.ShowModal() == wx.ID_OK:
            self.logger.info(_('Update - new release installed - restart %s') % TITLE)
            self.mainmenu.Enable(self.ID_INSTALL, False)    
    #-------------------------------------------------------------------------------------------

    def onWebsite(self, event):
        try:
            webbrowser.open(URL_WEBSITE)
        except:
            dlg = wx.MessageDialog(self, _('cannot connect to the NofyHist website!'), TITLE + _(' - browse website'), wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def onHandbook(self, event):
        """ show Html help """
        try:
            webbrowser.open(URL_HELPWEBPAGE)
        except:
            dlg = wx.MessageDialog(self, _('cannot connect to the NofyHist website!'), TITLE + _(' - browse help page'), wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def onInitIni(self, event):
        info = TITLE + '.ini file settings were corrupted.\nA new .ini file with default \
settings has been created.\nThe old .ini file is saved as ' + \
self.config['Version']['inifilename'].replace('.ini', '.ini.update')        
        convertwin = mumapopup.UpdatePopup(self, info)
        pos = self.notebook.ClientToScreen((0,0))  # pos is the left top edge of the win
        sz = self.notebook.GetSize()               # sz is the size of the win
        convertwin.Position((pos[0] + sz[0] / 2 - 150, pos[1] - 50 - sz[1] / 2), (0, sz[1]))
        convertwin.Popup()
    #-------------------------------------------------------------------------------------------

    def onSysTray(self, event = None):
        if event != None:
            self.handleConfigData('w')          # save window position
        self.tbicon.minimizeToTaskbar(len(self.messageshistory.getMessages()))
    #-------------------------------------------------------------------------------------------

    def OnRestoreWindow(self, event):
        self.Show(True)
        self.Iconize(False)
        self.tbicon.RemoveIcon()
        self.Raise()       
    #-------------------------------------------------------------------------------------------------------
    
    def onCloseWindow(self, event = None):
        self.handleConfigData('w')
        self.logger.info(_('NofyHist shutdown\n'))
        logging.shutdown()
        self.Destroy()              # switch the light off
# - class NofyHistFrame(wx.Frame):------------------------------------------------------------------


class NofyHistApp(wx.App):
    def OnInit(self):
        inipath = HOMEDIR
        if len(sys.argv) > 1:
            argument = sys.argv[1].lower()
            if argument.startswith('-i='):
                inipath = sys.argv[1][3:]
        inifilename = os.path.join(inipath, TITLE + '.ini')
        config = ConfigObj(inifilename, encoding = 'UTF8')
        
        try:
            si = SingleInstance(TITLE)
        except:
            return True
                   
        frame = NofyHistFrame(None, -1, VERSION, si)
        frame.Show()
        return True
    #-------------------------------------------------------------------------------------------

    def OnExit(self):
        try:
            self.lock.release()
        except:
            pass                    # we did our best
# - class NofyHistApp(wx.App):------------------------------------------------------------------


    
