#!/usr/bin/env python
#-*- coding: UTF-8 -*-
 
import wx
from   .nofyhistmisc import *


class SysTrayIcon(wx.TaskBarIcon):
    """ Minimize program to system tray and restore - some useful options """
    TBMENU_RESTORE = wx.NewId()
    TBMENU_CLOSE = wx.NewId()
    TBMENU_PREFS = wx.NewId()
    TBMENU_HELP = wx.NewId()

    def __init__(self, frame, icontemplate, fontpath, title):
        wx.TaskBarIcon.__init__(self)
        self.frame = frame
        self.icontemplate = icontemplate
        self.title = title
        
        if PIL_INSTALLED:
            self.font33 = ImageFont.truetype(fontpath, 33)
            self.font31 = ImageFont.truetype(fontpath, 31)

        self.Bind(wx.EVT_TASKBAR_LEFT_DOWN, self.onRestore)
        self.Bind(wx.EVT_MENU, self.onRestore, id = self.TBMENU_RESTORE)
        self.Bind(wx.EVT_MENU, self.onTaskBarClose, id = self.TBMENU_CLOSE)
        self.Bind(wx.EVT_MENU, self.frame.onPreferences, id = self.TBMENU_PREFS)
        self.Bind(wx.EVT_MENU, self.frame.onHandbook, id = self.TBMENU_HELP)
    #-------------------------------------------------------------------------------------------

    def onRestore(self, event):
        self.frame.Iconize(False)
        self.frame.Show(True)
        self.RemoveIcon()
        self.frame.Raise()
    #-------------------------------------------------------------------------------------------

    def onTaskBarClose(self, event):
        self.RemoveIcon()
        self.frame.onCloseWindow(event)
    #-------------------------------------------------------------------------------------------

    def CreatePopupMenu(self):
        """
        This method is called by the base class when it needs to popup
        the menu for the default EVT_RIGHT_DOWN event.  Just create
        the menu how you want it and return it from this function,
        the base class takes care of the rest.
        """
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.TBMENU_RESTORE, _('Restore NofyHist'))
        item.SetBitmap(loadIcon('restore.png'))        
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.TBMENU_PREFS, _('Preferences'))
        item.SetBitmap(loadIcon('prefs.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.TBMENU_HELP, _('Help'))
        item.SetBitmap(loadIcon('help.png'))
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.TBMENU_CLOSE, _('Exit NofyHist'))
        item.SetBitmap(loadIcon('exit.png'))
        menu.AppendItem(item)
        return menu
    #-------------------------------------------------------------------------------------------

    def createDigitIcon(self, digit):
        if PIL_INSTALLED:
            pilimage = self.icontemplate.copy()
            draw = ImageDraw.Draw(pilimage)
            if digit < 10:
                draw.text((15,6), str(digit), fill=(30,0,30), font=self.font33)
            else:
                draw.text((7,8), str(digit), fill=(30,0,30), font=self.font31)
            image = wx.EmptyImage(pilimage.size[0], pilimage.size[1])
            image.SetData(pilimage.convert('RGB').tobytes())
            image.Rescale(SCALE, SCALE, wx.IMAGE_QUALITY_NORMAL)
            icon = wx.IconFromBitmap(image.ConvertToBitmap())
            return icon
        else:
            return self.icontemplate
    #-------------------------------------------------------------------------------------------

    def setTitle(self, title, count):
        self.title = title
        if not self.frame.IsShown(): #self.frame.IsIconized(): funktioniert in wx 3.0 nicht
            icon = self.createDigitIcon(count)
            self.SetIcon(icon, self.title)
    #-------------------------------------------------------------------------------------------

    def minimizeToTaskbar(self, count):
        icon = self.createDigitIcon(count)
        self.SetIcon(icon, self.title)
        self.frame.Iconize(True)
        self.frame.Show(False)
# - class SysTrayIcon(wx.TaskBarIcon):----------------------------------------------------------
